#! /usr/bin/env python
"""Interface to get a configured logger"""

import importlib


def getLogger(config=None):
    """
    Returns a logger object of the type specified.

    :param config: Dictionary of values to configure the desired logger object.

    For example, to use the default python logger, you can
    either simply pass None, or specify any details supported by
    the toolchest default driver for the python logger.  You
    would specify the default logger (with custom loglevel) with:
    ::

      {'driver':'toolchest.logging.drivers.default',
      'root':{'level':DEBUG}}

    The toolchest logging wrapper code specifies a toplevel element in the
    config dict of 'driver', pointing at the driver that will load the
    desired configuration and return a logger to use.

    Each driver must implement getLogger, so we can invoke it the same way from
    this interface. In theory, drivers could be released as separate modules.

    """
    if not config:
        config = _default_config()
    try:
        module = importlib.import_module(config.get('driver'))
        return module.getLogger(config)
    except ModuleNotFoundError as e:
        logger = getLogger(_default_config())
        logger.error('Loading the default driver, {0}'.format(e.msg))
        return logger


def _default_config():
    return {'driver': 'toolchest.logging.drivers.default'}



#!/usr/bin/env python

# (C) 2018 Red Hat, Inc.
# License: Same as librpm: LGPL version 2


# Python copy of rpmvercmp() from rpmvercmp.c
#
# This is _intentionally_ as verbose as possible in order to most
# closely match rpmvercmp.c from librpm.  It is *easy* to make this
# more "pythonic", but then it would be more difficult to track/
# update changes and keeping this implementation bug-for-bug
# compatible with rpmvercmp.c's implementation.
def rpmvercmp(left, right):
    """
    Determines which label string has a higher value.

    This is for purposes of determining with version of a build is newer, or if
    they are equivalent based on string comparison.

    :param left: The left value to compare. Must be a string.
    :param right: The right value to compare. Must be a string.
    :returns: -1, 0, or 1 if the left value is less than, equal to,
            or greater than, the right value, respectively
    """

    # Note: Enhancement, for Pungi
    if left == 'None':
        left = None
    if right == 'None':
        right = None
    if left is None and right is None:
        return 0
    if left is None and right is not None:
        return -1
    if left is not None and right is None:
        return 1
    # # End Enhancement # #

    if not isinstance(left, str):
        raise ValueError('left is not a string')
    if not isinstance(right, str):
        raise ValueError('right is not a string')

    if left == right:
        return 0

    llen = len(left)
    rlen = len(right)
    li = 0
    ri = 0
    isnum = False

    while li < llen or ri < rlen:
        while (li < llen) and (not left[li].isalnum()) and left[li] != '~':
            li = li + 1
        while (ri < rlen) and (not right[ri].isalnum()) and right[ri] != '~':
            ri = ri + 1

        # The C code will compare vs. \0 here in some cases, which we can't
        # do directly, so watch for running past the end of the string and
        # assign a non-tilde if needed for this block. We need two try blocks
        # since we need to check these values independently.
        left_str = ''
        right_str = ''
        try:
            left_str = left[li]
        except IndexError:
            pass
        try:
            right_str = right[ri]
        except IndexError:
            pass

        if (left_str == '~' or right_str == '~'):
            if (left_str != '~'):
                return 1
            if (right_str != '~'):
                return -1
            li = li + 1
            ri = ri + 1
            continue

        if (li >= llen or ri >= rlen):
            break

        lp = li
        rp = ri

        if (left[lp].isdigit()):
            while (lp < llen and left[lp].isdigit()):
                lp = lp + 1
            while (rp < rlen and right[rp].isdigit()):
                rp = rp + 1
            isnum = True
        else:
            while (lp < llen and left[lp].isalpha()):
                lp = lp + 1
            while (rp < rlen and right[rp].isalpha()):
                rp = rp + 1
            isnum = False

        str1 = left[li:lp]
        str2 = right[ri:rp]

        if len(str1) == 0:
            return -1
        if len(str2) == 0:
            if isnum:
                return 1
            else:
                return -1

        if isnum:
            # Clear leading zeroes, if any
            str1 = str(int(str1))
            str2 = str(int(str2))

            if len(str1) > len(str2):
                return 1
            if len(str2) > len(str1):
                return -1

        if str1 != str2:
            if str1 < str2:
                return -1
            else:
                return 1

        li = lp
        ri = rp

    if li >= llen and ri >= rlen:
        return 0

    if li >= llen:
        return -1
    return 1


# Python implementation of rpm.labelCompare that does not need
# the RPM C libraries/packages; logic is from rpmvercmp.c
# rpmVersionCompare() function.
def labelCompare(left, right):
    """
    Determines which label tuple has a higher value.

    This is for purposes of determining with version of a build is newer, or if
    they are equivalent taking into account epoch, version, release.

    :param left: The left value to compare. Must be a tuple of (epoch, version,
                 release) or (version, release).
    :param right: The right value to compare. Must be a tuple of (epoch,
                  version, release) or (version, release).
    :returns: -1, 0, or 1 if the left value is less than, equal to,
              or greater than, the right value, respectively.
    """

    if not isinstance(left, tuple):
        raise ValueError('left is not a tuple')
    if not isinstance(right, tuple):
        raise ValueError('right is not a tuple')

    epoch_l = 0
    epoch_r = 0
    if len(left) == 3:
        if left[0] != '':
            epoch_l = int(left[0])
        version_l = str(left[1])
        release_l = str(left[2])
    elif len(left) == 2:
        version_l = str(left[0])
        release_l = str(left[1])
    else:
        raise ValueError('left is not a tuple of 2 or 3')

    if len(right) == 3:
        if right[0] != '':
            epoch_r = int(right[0])
        version_r = str(right[1])
        release_r = str(right[2])
    elif len(right) == 2:
        version_r = str(right[0])
        release_r = str(right[1])
    else:
        raise ValueError('right is not a tuple of 2 or 3')

    if epoch_l < epoch_r:
        return -1
    if epoch_l > epoch_r:
        return 1

    rc = rpmvercmp(version_l, version_r)
    if rc != 0:
        return rc

    return rpmvercmp(release_l, release_r)



import pytest

from toolchest.validator.simple import validate_string


def test_returns_true_for_string():
    result = validate_string('farkle')
    assert result is True


def test_raises_with_non_string():
    with pytest.raises(ValueError, match='5 is not a string'):
        validate_string(5)
    with pytest.raises(ValueError, match='None is not a string'):
        validate_string(None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath('../'))


# -- Project information -----------------------------------------------------

project = u'toolchest'
copyright = u"2018, Red Hat, Inc"
author = u"Red Hat, Inc"

# -- General configuration ---------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.viewcode',
              'sphinx.ext.napoleon', 'myst_parser']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = ['.rst', '.md']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output -------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Theme options are theme-specific and customize the look and feel of a
# theme further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# -- Custom tooling ----------------------------------------------------------
import os
import subprocess

def setup(app):
    app.connect('builder-inited', build_changelog)

def build_changelog(app):
    docs_dir = os.getcwd()
    os.chdir('../')
    root_dir = os.getcwd()
    cmd = ['bash', f"{root_dir}/tooling/build_changelog", root_dir]
    change_proc = subprocess.run(cmd, capture_output=True, text=True)
    print(f"change stderr returns: {change_proc.stderr}")
    print(f"change stdout returns: {change_proc.stdout}")
    os.chdir(docs_dir)



#!/usr/bin/env python

import re


def list_or_splitstr(arg):
    if type(arg) is list:
        return arg
    if type(arg) is not str:
        raise ValueError('arg is not a string or list')

    return [x for x in re.split('[\t\n, ]', arg) if x != '']


def split_file(arg):
    with open(arg, 'r') as f:
        text = f.read()
        return list_or_splitstr(text)
    return None


def regex_chars(arg):
    s = set('[]()|.*+')
    if any((c in s) for c in arg):
        return True
    return False


def regex_match(pattern, arg):
    if re.match(pattern, arg) is not None:
        return True
    return False



"""
Test the utility methods
"""

import pytest  # NOQA
from toolchest.rpm.rpmvercmp import labelCompare, rpmvercmp


class TestRpmvercmp(object):

    def test_equal(self):
        assert rpmvercmp('1.2', '1.2') == 0
        assert rpmvercmp('a', 'a') == 0

    def test_basic(self):
        assert rpmvercmp('1.1', '1.2') == -1
        assert rpmvercmp('1.2', '1.1') == 1
        assert rpmvercmp('1.2a', '1.2') == 1
        assert rpmvercmp('1.2', '1.2a') == -1
        assert rpmvercmp('1.2', '1.1abcde') == 1
        assert rpmvercmp('1', '11') == -1
        assert rpmvercmp('11', '1') == 1
        assert rpmvercmp('a', 'b') == -1
        assert rpmvercmp('b', 'a') == 1

    def test_lead_zeroes(self):
        assert rpmvercmp('1.2.2031', '1.2.02031') == 0
        assert rpmvercmp('01.02.0000002031', '1.2.2031') == 0
        assert rpmvercmp('1.2.2031', '1.2.00002030') == 1
        assert rpmvercmp('1.2.2030', '1.2.00002031') == -1

    def test_excessive_version_length(self):
        assert(rpmvercmp('9999999999999999999999999999999999999999999',
                         '9999999999999999999999999999999999999999999') == 0)
        assert(rpmvercmp('9999999999999999999999999999999999999999999',
                         '9999999999999999999999999999999999999999998') == 1)
        assert(rpmvercmp('9999999999999999999999999999999999999999998',
                         '9999999999999999999999999999999999999999999') == -1)

    def test_weird_versions(self):
        a = 'a'
        b = '1'
        assert rpmvercmp(a, b) == -1
        assert rpmvercmp(b, a) == 1

        a = '1.01.001'
        b = '1.1.1'
        assert rpmvercmp(a, b) == 0

        a = '~1'
        assert rpmvercmp(a, b) == -1
        assert rpmvercmp(b, a) == 1

        a = '1~1'
        assert rpmvercmp(a, b) == -1
        assert rpmvercmp(b, a) == 1

        a = '----'
        b = '-'
        assert rpmvercmp(a, b) == 0

        a = '1-'
        b = '1'
        assert rpmvercmp(a, b) == 0

        a = '1-'
        b = '-'
        assert rpmvercmp(a, b) == 1
        assert rpmvercmp(b, a) == -1

    def test_bad_types(self):
        with pytest.raises(ValueError):
            rpmvercmp({}, '1.0')
        with pytest.raises(ValueError):
            rpmvercmp('1.0', {})


class TestLabelCompare(object):

    def test_basic(self):
        a = ('0', '6.0.0', '2.el7ost')
        b = ('0', '6.0.1', '2.el7ost')
        assert labelCompare(a, b) == -1
        assert labelCompare(a, a) == 0
        assert labelCompare(b, a) == 1
        assert labelCompare(b, b) == 0

    def test_vr_only(self):
        # no epoch => epoch = 0
        a = ('6.0.0', '2.el7ost')
        b = ('6.0.1', '2.el7ost')
        assert labelCompare(a, b) == -1
        assert labelCompare(a, a) == 0
        assert labelCompare(b, a) == 1
        assert labelCompare(b, b) == 0

    def test_zero_epoch(self):
        # 0 and no epoch are the same
        a = ('0', '6.0.0', '2.el7ost')
        b = ('6.0.1', '2.el7ost')
        assert labelCompare(a, b) == -1
        assert labelCompare(a, a) == 0
        assert labelCompare(b, a) == 1
        assert labelCompare(b, b) == 0

        # librpm seems to set '' to 0
        a = ('', '1.0', '1')
        b = ('1.0', '1')
        c = (0, '1.0', '1')
        d = ('0', '1.0', '1')
        assert labelCompare(a, b) == 0
        assert labelCompare(a, c) == 0
        assert labelCompare(a, d) == 0
        assert labelCompare(b, c) == 0
        assert labelCompare(b, d) == 0
        assert labelCompare(c, d) == 0

    def test_eq_epoch(self):
        a = ('1', '6.0.0', '2.el7ost')
        b = ('1', '6.0.1', '2.el7ost')
        assert labelCompare(a, b) == -1
        assert labelCompare(a, a) == 0
        assert labelCompare(b, a) == 1
        assert labelCompare(b, b) == 0

    def test_neq_epoch(self):
        # a always wins.
        a = ('1', '6.0.0', '2.el7ost')
        b = ('0', '6.0.1', '2.el7ost')
        assert labelCompare(a, b) == 1
        assert labelCompare(a, a) == 0
        assert labelCompare(b, a) == -1
        assert labelCompare(b, b) == 0

    def test_bad_types(self):
        a = '1:6.0.0-2.el7ost'
        b = 1
        with pytest.raises(ValueError):
            labelCompare(a, b)
        with pytest.raises(ValueError):
            labelCompare(b, a)

    def test_bad_length(self):
        a = ('1', '6.0.0', '2.el7ost', 1)
        b = ('0', '6.0.1', '2.el7ost')
        with pytest.raises(ValueError):
            labelCompare(a, b)
        with pytest.raises(ValueError):
            labelCompare(b, a)

    def test_bad_epoch(self):
        a = ('abcde', '1.0', '1')
        b = ('1.0', '1')
        with pytest.raises(ValueError):
            labelCompare(a, b)

    def test_weird_versions(self):
        a = ('0', 'a', '1')
        b = ('0', '1', '1')
        assert labelCompare(a, b) == -1
        assert labelCompare(b, a) == 1

        a = ('0', '1.01.001', '1')
        b = ('0', '1.1.1', '1')
        assert labelCompare(a, b) == 0

        a = ('0', '~1', '1')
        b = ('0', '1', '1')
        assert labelCompare(a, b) == -1
        assert labelCompare(b, a) == 1

        a = ('0', '1~1', '1')
        b = ('0', '1', '1')
        assert labelCompare(a, b) == -1
        assert labelCompare(b, a) == 1



#!/usr/bin/env python
import hashlib

from toolchest.strutil import list_or_splitstr
from toolchest.validator.simple import validate_string
from toolchest.validator.base import BaseValidator


def generate_hash(list_to_hash, validator=validate_string):
    """
    Takes a list of strings in any order, sorts them for consistency, and
    generates a cryptographic hash to provide a unique representation of this
    combination of strings.

    :param list list_to_hash: List of Strings to turn into a hash
    :param func validator: A function to use to validate the format of items
                           in the list of Strings. This function should throw
                           a ValueError if the pattern does not match.
    :return: A String that is a hash of the passed in list.
    :raises ValueError: If the passed in list cannot be validated with the
                        passed in validator, a ValueError is thrown.
    """
    hash_obj = hashlib.sha3_512()
    list_to_hash = list_or_splitstr(list_to_hash)
    list_to_hash.sort()
    for item in list_to_hash:
        BaseValidator.validate(validator, item)
        hash_obj.update(bytes(item, 'utf-8'))
    return hash_obj.hexdigest()



#!/usr/bin/env python

import os
import itertools
import sys
import time
import signal

display_color = True
_done = True

HILIGHT = '[1m'
NORMAL = '[0m'

COLORS = {'red': '\033[31m',
          'yellow': '\033[33m',
          'blue': '\033[34m',
          'green': '\033[32m',
          'default': '\033[0m'}


def ask_yesno(message):
    ret = input(message + ' [y/N]? ')
    if ret in ('y', 'Y', 'yes', 'YES'):
        return True
    return False


def color_string(string, color):
    if display_color is not True:
        return string

    ret_string = ''
    if color in COLORS:
        ret_string = '{0}{1}{2}'.format(COLORS[color],
                                        string,
                                        COLORS['default'])
    else:
        ret_string = string

    return ret_string


def usleep(msec):
    time.sleep(msec / 1000000.0)


def exithandler(sig, frame):
    global _done
    _done = True


class Spinner(object):

    def __init__(self, startmsg='', endmsg=''):
        global _done

        self._endmsg = endmsg
        sys.stdout.write(startmsg)
        sys.stdout.flush()

        # No-op when redirecting
        if not sys.stdout.isatty():
            self._pid = -1
            return

        self._pid = os.fork()
        if self._pid:
            return
        signal.signal(signal.SIGTERM, exithandler)
        _done = False
        self._spinner = itertools.cycle('-/|\\')
        sys.stdout.write('\\')
        while not _done:
            sys.stdout.write('\b' + next(self._spinner))
            sys.stdout.flush()
            usleep(100000)
        sys.stdout.write('\b')
        sys.stdout.flush()
        exit(0)

    def __del__(self):
        if self._pid > 0:
            os.kill(self._pid, signal.SIGTERM)
            os.wait()
        if self._pid != 0:
            sys.stdout.write(self._endmsg)
            sys.stdout.flush()

    def __enter__(self):
        pass

    def __exit__(self, t, v, traceback):
        if t is not None:
            raise


# Object wrapper around python-prettytable. Fakes it
# if prettytable is not installed.
class PrettyTable(object):
    def __init__(self, column_names):
        self.ugly = True
        self.cols = column_names
        self.rows = []
        self.align = None

        try:
            import prettytable
            self.pt = prettytable.PrettyTable(column_names)
            self.ugly = False
            return
        except ImportError:
            pass

    def add_row(self, row):
        if not self.ugly:
            return self.pt.add_row(row)

        if len(row) != len(self.cols):
            msg = 'Row length is {0}, but should be {1}'
            raise ValueError(msg.format(str(len(row)), str(len(self.cols))))
        self.rows.append(row)

    def get_string(self):
        if not self.ugly:
            try:
                if self.align is not None:
                    self.pt.align = self.align
            except AttributeError as e:
                print(e)
                pass
            return self.pt.get_string()

        s = ''
        s = s + '\t'.join([str(q) for q in self.cols]) + '\n'
        for r in self.rows:
            s = s + '\t'.join([str(q) for q in r]) + '\n'
        return s


def box(msg):
    print('+-' + ('-' * len(msg)) + '-+')
    print('| ' + msg + ' |')
    print('+-' + ('-' * len(msg)) + '-+')


def line(msg):
    return '-' * len(msg)



"""Test logging functionality"""

import logging
from unittest.mock import MagicMock
from toolchest.logging import logger


def test_gets_default_logger():
    """
    GIVEN we request a logger with the default toolchest logger
    WHEN we specify the root log level
    THEN we get back a default python logger, with that level set.
    """
    mylogger = logger.getLogger({'driver': 'toolchest.logging.drivers.default',
                                'root': {'level': 'DEBUG'}})
    assert isinstance(mylogger, logging.Logger)
    assert mylogger.getEffectiveLevel() == logging.DEBUG


def test_no_logger_specified():
    """
    GIVEN we request a logger without specifying any details
    THEN we get back a default python logger with log level = WARNING
    """
    mylogger = logger.getLogger()
    assert isinstance(mylogger, logging.Logger)
    assert mylogger.getEffectiveLevel() == logging.WARNING


def test_driver_not_found():
    """
    GIVEN we request a logger with a driver that cannot be found
    THEN we get back a default python logger with log level = WARNING
    """
    logging.Logger.error = MagicMock()
    mylogger = logger.getLogger({'driver': 'foo.bar.baz'})

    mylogger.error.assert_called_with(
        "Loading the default driver, No module named 'foo'")
    assert isinstance(mylogger, logging.Logger)
    assert mylogger.getEffectiveLevel() == logging.WARNING



def validate_string(str_to_validate):
    """
    Validates that the value passed in is a string.

    :param str str_to_validate: A value to validate
    :return: Returns True if pattern matches
    :raises ValueError: If the passed in item is not a string, a ValueError
            is thrown.
    """

    if type(str_to_validate) is not str:
        raise ValueError(f"{str_to_validate} is not a string")
    return True



#!/usr/bin/env python

""" Tests for the yaml module """

import os.path
import subprocess

import pytest

from toolchest.yaml import parse, write


def test_simple_string():
    expected = {'a': 1}
    input_str = '---\na: 1'
    actual = parse(input_str)
    assert actual == expected


def test_bad_string():
    expected = {}
    input_str = ':'
    actual = parse(input_str)
    assert actual == expected


def test_file_read(datadir):
    file_to_load = os.path.join(datadir, 'sample.yml')
    expected = {'a': 1,
                'b': 2,
                'c': [7, 8, 9]}
    file_handle = open(file_to_load, 'r')
    actual = parse(file_handle.read())
    assert actual == expected


def test_bad_write():
    data = {'a': 'apple', 'b': 'banana'}
    output_file = '/bad/path/data.yml'
    with pytest.raises(FileNotFoundError):
        write(output_file, data)
    assert not os.path.exists(output_file)


def test_good_write(tmp_path):
    expected = {'a': 'apple', 'b': 'banana'}
    output_file = os.path.join(tmp_path, 'generated.yml')
    write(output_file, expected)
    with open(output_file, 'r') as check_fn:
        parsed = parse(check_fn.read())
        assert parsed == expected


def test_read_write_roundtrip(datadir, tmp_path):
    output_file = os.path.join(tmp_path, 'generated.yml')
    input_file = os.path.join(datadir, 'sample.yml')
    with open(input_file, 'r') as input_fh:
        output_data = parse(input_fh.read())
        write(output_file, output_data)
        # If an exception is raised the files contain a diff and the test
        # will fail. We want to make sure we can write out in the same format
        # that we read in.
        subprocess.check_call(['diff', input_file, output_file])



#! /usr/bin/env python
""" Yaml related tools """

import yaml


class IndentDumper(yaml.Dumper):
    """ Custom yaml dumper to ensure elements are indented """
    def increase_indent(self, flow=False, indentless=False):
        # We can't set indentless from the yaml.dump call to control behavior.
        # When this method is called, indentless is being set to True so it
        # can't be used in the call to super. We the super call to always be
        # False.
        return super().increase_indent(flow=flow, indentless=False)


def parse(yaml_data, logger=None):
    """
    Safely parse yaml data

    :param yaml_data: A string of yaml data to be parsed
    :param logger: Optional logger for potential errors
    :return: A dictionary formed out of the yaml data
    """
    ret_data = {}
    try:
        ret_data = yaml.safe_load(yaml_data)
    except yaml.YAMLError as exc:
        msg = 'Error in yaml data'
        if hasattr(exc, 'problem_mark'):
            error_str = 'Yaml Error at: (line: {0}, column: {1})'
            msg = error_str.format(exc.problem_mark.line + 1,
                                   exc.problem_mark.column + 1)
        if logger is not None:
            logger.error(msg)
        else:
            print(msg)
    return ret_data


def write(filename, yaml_data, sort_keys=False):
    """
    Write out yaml data to a file

    :param filename: The name of the file to write the data
    :param yaml_data: A string of yaml data to be parsed
    :param sort_keys: Have the yaml data's keys sorted on write
    :return: None
    """

    # open the file for write, if it does not exist create it.
    with open(filename, 'w') as output_file:
        # write yaml header
        output_file.write('---\n\n')
        # write the actual data
        yaml.dump(yaml_data, output_file, Dumper=IndentDumper,
                  default_flow_style=False, sort_keys=sort_keys)



#!/usr/bin/env python


# For calling cmdline functions w/o using
# argparse
class GenericArgs(dict):
    def __getattr__(self, name):
        if (name in self):
            return dict.__getitem__(self, name)
        return None
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__



import pytest

from toolchest.validator.base import BaseValidator
from toolchest.validator.simple import validate_string


def misbehaving_validator(item_to_validate):
    return "I didn't read the docs!"


def test_returns_true_with_good_validator():
    result = BaseValidator.validate(validate_string, 'my string')
    assert result is True


def test_throws_value_error_on_bad_input():
    with pytest.raises(ValueError, match='5 is not a string'):
        BaseValidator.validate(validate_string, 5)


def test_noncompliant_validator():
    with pytest.raises(ValueError, match='5 failed validation'):
        BaseValidator.validate(misbehaving_validator, 5)



# -*- coding: utf-8 -*-
"""
Methods related to introspection of a class
"""


def check_if_property(instance, prop):
    """
    Check if a given value is a property on the specified class instance

    :param instance: An instance of some class
    :param prop: A value to check
    :return: True or False
    """
    return isinstance(getattr(instance.__class__, prop), property)


def build_attrs(instance):
    """
    Build a dict of attributes on a given instance

    :param instance: An instance of some class
    :return: Dict of all object attributes that are properties
    """
    return {prop: instance.__getattribute__(prop)
            for prop in dir(type(instance)) if
            check_if_property(instance, prop)}


def build_repr(instance):
    """
    Generic way to implement a __repr__ method for classes using properties

    :param instance: An instance of some class
    :return: String that can be eval'ed to an object
    """
    return f"{type(instance).__name__}({build_attrs(instance)})"



class BaseValidator():

    @staticmethod
    def validate(validator, item_to_validate):
        """
        A method to allow calling of any validator, with checking to only
        return True or an error, so that the validator cannot inject code
        into the caller.

        :param func validator: A function to use to validate the format of an
                            item. This function should throw a ValueError if
                            the pattern does not match.
        :param obj item_to_validate: A value to validate
        :return: Returns True if pattern matches
        :raises ValueError: If the item cannot be validated with the
                            validator, a ValueError is thrown.
        """

        is_valid = validator(item_to_validate)
        if is_valid is True:
            return True
        else:
            # This should only happen if the validator tries to do something
            # besides returning True or ValueError.  The purpose is to
            # prevent malicious code being injected.
            msg = f"{item_to_validate} failed validation, "
            msg += f"{validator} does not implement correct interface"
            raise ValueError(msg)



from toolchest.filesystem.git import generate_gitreview


def test_gen_single_gitreview(tmp_path):
    """
    GIVEN a list of .gitreview fields including a defaultrebase value
    WHEN generate_gitreview is called
    THEN a .gitreview file is written with the specified fields
    AND the given defaultrebase value is written to the file
    """
    path = tmp_path
    host = "random_host_str"
    host_port = "1234"
    project = "test_project"
    branch = "test_branch_name"
    remote = "test_remote"
    rebase = 0

    generate_gitreview(path,
                       project,
                       host,
                       host_port,
                       branch,
                       remote,
                       rebase)

    with open(f"{path}/.gitreview", "r") as file_opened:
        correct_output = [
            "[gerrit]\n",
            "host=random_host_str\n",
            "port=1234\n",
            "project=test_project.git\n",
            "defaultbranch=test_branch_name\n",
            "defaultremote=test_remote\n",
            "defaultrebase=0\n"
        ]
        for line in correct_output:
            assert line == file_opened.readline()


def test_gen_single_default_rebase_gitreview(tmp_path):
    """
    GIVEN a list of .gitreview fields without defaultrebase given
    WHEN generate_gitreview is called
    THEN a .gitreview file is written with the specified fields
    AND defaultrebase is given a value of 1 in the .gitreview file
    """
    path = tmp_path
    host = "random_host_str"
    host_port = "1234"
    project = "test_project"
    branch = "test_branch_name"
    remote = "test_remote"

    generate_gitreview(path,
                       project,
                       host,
                       host_port,
                       branch,
                       remote)

    with open(f"{path}/.gitreview", "r") as file_opened:
        correct_output = [
            "[gerrit]\n",
            "host=random_host_str\n",
            "port=1234\n",
            "project=test_project.git\n",
            "defaultbranch=test_branch_name\n",
            "defaultremote=test_remote\n",
            "defaultrebase=1\n"
        ]
        for line in correct_output:
            assert line == file_opened.readline()



import unittest

from toolchest.rpm.utils import split_filename, drop_epoch

# Handles DLRN versions
from toolchest.rpm.utils import labelCompare, dlrn_label_compare

# Compare vs. baseline
from toolchest.rpm.rpmvercmp import labelCompare as rpmLabelCompare

from toolchest.rpm.utils import cpaas_label_compare


class test_split_filename(unittest.TestCase):

    def test_normal(self):
        assert (split_filename('foo-1.2-1.f23.noarch.rpm') ==
                ('foo', '1.2', '1.f23', '', 'noarch'))
        assert (split_filename('foo-1.2-1.f23.noarch') ==
                ('foo', '1.2', '1.f23', '', 'noarch'))
        assert (split_filename('sos-3.2-36.el7ost.1') ==
                ('sos', '3.2', '36.el7ost.1', '', ''))
        assert (split_filename('sos-3.2-36.el7ost.x86_64') ==
                ('sos', '3.2', '36.el7ost', '', 'x86_64'))
        assert (split_filename('sos-3.2-36.el7ost.1.x86_64') ==
                ('sos', '3.2', '36.el7ost.1', '', 'x86_64'))
        assert (split_filename('sos-3.2-36.el7ost.1.x86_64.rpm') ==
                ('sos', '3.2', '36.el7ost.1', '', 'x86_64'))
        assert (split_filename('sos-3.2-36.el7ost.1.src.rpm') ==
                ('sos', '3.2', '36.el7ost.1', '', ''))
        fn = 'openstack-nova-15.0.1-0.20170222170803.10a32dd.el7ost'
        split_fn = ('openstack-nova',
                    '15.0.1',
                    '0.20170222170803.10a32dd.el7ost',
                    '',
                    '')
        assert (split_filename(fn) == split_fn)
        assert (split_filename('openstack-nova-1:15.0.1-1.el7ost') ==
                ('openstack-nova', '15.0.1', '1.el7ost', '1', ''))

    def test_lead_epoch(self):
        assert (split_filename('2:foo-1.2-1.f23.noarch') ==
                ('foo', '1.2', '1.f23', '2', 'noarch'))
        # error: but, use lead epoch
        assert (split_filename('2:foo-3:1.2-1.f23.noarch') ==
                ('foo', '1.2', '1.f23', '2', 'noarch'))
        assert (split_filename('1:openstack-nova-15.0.1-1.el7ost.noarch') ==
                ('openstack-nova', '15.0.1', '1.el7ost', '1', 'noarch'))
        assert (split_filename('1:openstack-nova-15.0.1-1.el7ost') ==
                ('openstack-nova', '15.0.1', '1.el7ost', '1', ''))

    def test_epoch(self):
        assert (split_filename('foo-3:1.2-1.f23.noarch') ==
                ('foo', '1.2', '1.f23', '3', 'noarch'))

    def test_no_component(self):
        assert (split_filename('1.2-1.f23.noarch') ==
                ('', '1.2', '1.f23', '', 'noarch'))
        assert (split_filename('1.2-1.f23') ==
                ('', '1.2', '1.f23', '', ''))
        assert (split_filename('1.2-1') ==
                ('', '1.2', '1', '', ''))
        assert (split_filename('1.2') ==
                ('', '1.2', '', '', ''))

    def test_absurd(self):
        whole_filename = '1:aarch64-ppc64le-i686-0.1-1.el7ost.src.rpm'
        split_parts = ('aarch64-ppc64le-i686', '0.1', '1.el7ost', '1', '')
        assert (split_filename(whole_filename) == split_parts)
        fn = 'm-e-g-a-f-o-o-1.0-36.123.1333.elite.omg.long.el7ost.1.noarch.rpm'
        split_fn = ('m-e-g-a-f-o-o',
                    '1.0',
                    '36.123.1333.elite.omg.long.el7ost.1',
                    '',
                    'noarch')
        assert (split_filename(fn) == split_fn)

    def test_labelCompare(self):

        a = ('0', '6.0.0', '2.el7ost')
        b = ('0', '6.0.1', '0.20170222164853.el7ost')
        c = ('0', '6.0.0', '0.20170217223245.0rc1.el7ost')
        d = ('0', '6.1.10', '0.20170222164853.el7ost')
        e = ('0', '6.1.9', '1.el7ost')
        f = ('0', '6.0.0', '0.2.0rc2.el7ost')
        new_dlrn = ('0', '6.1.1', '0.2.20170217063119.ad33b59.el7ost')

        self.assertEqual(rpmLabelCompare(a, b),
                         labelCompare(a, b))

        # Two dlrn builds should be compared the same
        self.assertEqual(rpmLabelCompare(b, c),
                         labelCompare(b, c))

        # One dlrn build and one not with same version
        # should not matter
        self.assertEqual(rpmLabelCompare(a, c),
                         labelCompare(a, c))
        self.assertEqual(rpmLabelCompare(new_dlrn, d),
                         labelCompare(new_dlrn, d))
        self.assertEqual(rpmLabelCompare(new_dlrn, e),
                         labelCompare(new_dlrn, e))
        self.assertEqual(rpmLabelCompare(new_dlrn, b),
                         labelCompare(new_dlrn, b))
        self.assertEqual(rpmLabelCompare(d, e),
                         labelCompare(d, e))

        # Release candidate DLRN builds > dlrn builds
        # e.g. 0.2.0rc1 needs to be > 0.2348349839021890.abc444
        self.assertEqual(rpmLabelCompare(b, f),
                         labelCompare(b, f))

        # known prior test cases that were broken prior to previous
        #  dlrn_label_compare commits
        a = ('0', '1.2.0', '0.20191009110244.6090753.el8ost')
        b = ('0', '1.1.0', '1.20201113133400.6e1ba65.el8ost')
        self.assertEqual(rpmLabelCompare(a, b), labelCompare(a, b))
        self.assertEqual(rpmLabelCompare(a, b), dlrn_label_compare(a, b)[0])

        a = ('0', '2.3.2', '0.20191004134845.41e2a2b.el8ost')
        b = ('0', '2.3.1', '1.20201113163517.41e2a2b.el8ost')
        self.assertEqual(rpmLabelCompare(a, b), labelCompare(a, b))
        self.assertEqual(rpmLabelCompare(a, b), dlrn_label_compare(a, b)[0])

    def test_cpaas_label_compare(self):

        a = ('0', '6.0.0', '3.el7ost.0')
        b = ('0', '6.0.0', '3.el7ost.1')
        c = ('0', '6.0.1', '1.20170.0rc1.el7ost')
        d = ('0', '6.0.1', '1.20170.0rc1.el7ost.1')
        e = ('0', '6.1.0', '1.20170.0rc1.el7ost')
        f = ('', '6.1.0', '1.20170.0rc1.el7ost')

        self.assertEqual(cpaas_label_compare(a, b), 0)
        self.assertEqual(cpaas_label_compare(b, c), -1)
        self.assertEqual(cpaas_label_compare(c, d), 0)
        self.assertEqual(cpaas_label_compare(d, e), -1)
        self.assertEqual(cpaas_label_compare(e, f), 0)

        # if there's no divider, revert to normal label compare
        a = ('0', '6.0.0', '2')
        b = ('0', '6.0.0', '3')

        # Ignore divider if the main version changed
        c = ('0', '6.0.0', '2.el7ost.0')
        d = ('0', '6.0.0', '3.el7ost.0')
        e = ('0', '6.0.0', '3.el7ost')

        self.assertEqual(cpaas_label_compare(a, b), -1)
        self.assertEqual(cpaas_label_compare(c, d), -1)
        self.assertEqual(cpaas_label_compare(c, e), -1)


class test_drop_epoch(unittest.TestCase):

    def test_no_epoch(self):
        assert (drop_epoch('foo-1.2-1.f23.noarch.rpm') ==
                'foo-1.2-1.f23.noarch')
        assert (drop_epoch('foo-1.2-1.f23.noarch') ==
                'foo-1.2-1.f23.noarch')

    def test_epoch(self):
        assert (drop_epoch('foo-0:1.2-1.f23.noarch.rpm') ==
                'foo-1.2-1.f23.noarch')
        assert (drop_epoch('foo-1:1.2-1.f23.noarch') ==
                'foo-1.2-1.f23.noarch')
        assert (drop_epoch('foo-99999:1.2-1.f23.noarch') ==
                'foo-1.2-1.f23.noarch')
        assert (drop_epoch('0:foo-1.2-1.f23.noarch.rpm') ==
                'foo-1.2-1.f23.noarch')
        assert (drop_epoch('99999:foo-1.2-1.f23.noarch') ==
                'foo-1.2-1.f23.noarch')



import unittest

from toolchest.strutil import list_or_splitstr


class test_list_or_splitstr(unittest.TestCase):

    def test_identity(self):
        self.assertEqual(list_or_splitstr([1, 2]), [1, 2])

    def test_oops(self):
        with self.assertRaises(ValueError):
            list_or_splitstr({})
        with self.assertRaises(ValueError):
            list_or_splitstr(1)
        with self.assertRaises(ValueError):
            list_or_splitstr(None)

    def test_normal(self):
        self.assertEqual(list_or_splitstr('a b c'), ['a', 'b', 'c'])
        self.assertEqual(list_or_splitstr('a,b c'), ['a', 'b', 'c'])
        self.assertEqual(list_or_splitstr('a, b, c'), ['a', 'b', 'c'])
        self.assertEqual(list_or_splitstr('''a
    b        c'''), ['a', 'b', 'c'])
        self.assertEqual(list_or_splitstr('a	b	c'), ['a', 'b', 'c'])



"""
Test introspection methods
"""

import pytest  # NOQA
from toolchest.introspect import build_repr, build_attrs, check_if_property


class SomeClass(object):

    def __init__(self, key1=None):
        self._key1 = key1

    @property
    def key1(self):
        return self._key1


@pytest.fixture()
def an_object():
    return SomeClass()


class TestCheckIfProperty(object):

    def test_true_if_property(self, an_object):
        assert check_if_property(an_object, 'key1') is True

    def test_false_if_not_property(self, an_object):
        SomeClass.not_a_key = 'some value'
        assert check_if_property(an_object, 'not_a_key') is False


class TestBuildAttrs(object):

    def test_no_attrs(self, an_object):
        attrs_dict = build_attrs(an_object)
        assert an_object.key1 is None
        assert attrs_dict.get('key1') is None

    def test_returns_key_value_list(self):
        obj = SomeClass(key1="farkle")
        attrs_dict = build_attrs(obj)
        assert obj.key1 == 'farkle'
        assert attrs_dict.get('key1') == 'farkle'


class TestBuildRepr(object):

    def test_returns_string(self):
        obj = SomeClass(key1="farkle")
        obj_str = build_repr(obj)
        expected = "SomeClass({'key1': 'farkle'})"
        assert obj_str == expected



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

import os

from setuptools import setup


def check_if_scm():
    if os.environ.get('SCM_NO_LOCAL_SCHEME'):
        return "no-local-version"
    else:
        return "node-and-date"


setup(
    use_scm_version={'local_scheme': check_if_scm()}
)



import unittest

from toolchest.genericargs import GenericArgs


class test_genericargs(unittest.TestCase):

    def setUp(self):
        self.data = GenericArgs({'one': '1', 'two': '2'})

    def test_basic(self):
        assert self.data == {'one': '1', 'two': '2'}

    def test_items(self):
        assert self.data.one == '1'
        assert self.data.two == '2'

    def test_attr_not_found(self):
        # Returns None
        assert self.data.whatever is None

    def test_key_not_found(self):
        with self.assertRaises(KeyError):
            self.data['whatever'] == 0



#!/usr/bin/env python
import os


def generate_gitreview(path, project, host, port,
                       branch, remote, rebase=1):
    """Write a new .gitreview file to disk.

        :param str path: Directory where the file will be written
        :param str project: Name of the project
        :param str host: Gerrit host
        :param str port: Port for Gerrit host
        :param str branch: Git branch to use as defaultbranch
        :param str remote: Git remote to use as defaultremote
        :param int rebase: if 0, then the defaultrebase will be 0
                                 and the changes won't be rebased by default
                           if 1, then the defaultrebase will be 1
                                 and the changes will be rebased by default
    """
    with open(os.path.join(path, '.gitreview'), 'w') as fp:
        fp.write('[gerrit]\n')
        fp.write(f'host={host}\n')
        fp.write(f'port={port}\n')
        fp.write(f'project={project}.git\n')
        fp.write(f'defaultbranch={branch}\n')
        fp.write(f'defaultremote={remote}\n')
        fp.write(f'defaultrebase={rebase}\n')



#! /usr/bin/env python
"""Implementation of a driver that uses the standard python logger."""

import logging
import logging.config


def getLogger(config):
    """
    Returns a python stdlib logger

    :param config:  Dictionary of values meeting the criteria in the
                    `Configuration dictionary schema
                    <https://docs.python.org/3.7/library/logging.config.html#logging-config-dictschema>`_.

    """
    if 'version' not in config:
        config['version'] = 1
    if len(config) > 2:
        logging.config.dictConfig(config)
    logger = logging.getLogger(__name__)
    logger.setLevel(config.get('root', {}).get('level', logging.WARNING))
    return logger



#!/usr/bin/env python

import re
import warnings

from toolchest.rpm.rpmvercmp import labelCompare as rpmLabelCompare


def split_filename_fast(nvr):
    info = nvr.rsplit('-', 2)
    return (info[0], info[1], info[2])


# rpmUtils.miscutils and
# dnf.rpm.miscutils are bugged for several types of valid
# rpm names
def splitFilename(nvr):

    #
    # If we have a full rpm name, we'll have an arch
    # or src -
    #   foo-1.2-1.f23.noarch.rpm - 2 dots - arch exists
    #   foo-1.2-1.f23.noarch     - 2 dots - arch exists
    #   foo-1.2-1.noarch.rpm     - 1 dot, arch exists, no disttag
    #   foo-1.2-1.f23            - 1 dot... not 100% deterministic, but
    #                              likely .f23 is a disttag; very rare
    #   1.2-1.f23                - no name, just v, r
    #   1:1.2-1.f23              - no name, just e, v, r
    #   1:foo-1.2-1.f23          - no name, just e, v, r
    #
    # Junk .rpm if it exists
    if nvr[-4:] == '.rpm':
        nvr = nvr[:-4]

    #
    # Any field may be absent
    #
    n = ''
    v = ''
    r = ''
    e = ''
    a = ''

    # Determine arch in a deterministic way
    arches = ['noarch', 'x86_64', 'i386', 'i486', 'i586',
              'i686', 'ppc', 'ppc64', 'ppc64le', 's390',
              's390x', 'aarch64', 'src']
    arch_rx = r'\.(' + '|'.join(arches) + ')$'
    if re.search(arch_rx, nvr):
        x = nvr.rfind('.')
        a = nvr[x + 1:]
        if a == 'src':
            # src is not an arch; we just use it for
            # determinism
            a = ''
        nvr = nvr[:x]

    tmp = nvr

    #
    # version & release are after two last dashes, respectively
    # Get 'release' first
    #
    x = tmp.rfind('-')
    if x < 0:
        #
        # maybe someone provided just a version?
        #
        if tmp.rfind('.') >= 0:
            v = tmp
        else:
            n = tmp
        return n, v, r, e, a

    # Release
    r = tmp[x + 1:]
    tmp = tmp[:x]

    x = tmp.rfind('-')
    if x < 0:
        #
        # Someone just provided version-release?
        #
        v = tmp
        return n, v, r, e, a

    #
    # epoch can be in version or in rpm name
    # Check version first
    #
    version = tmp[x + 1:]
    tmp = tmp[:x]

    x = version.rfind(':')
    if x > 0:
        e = version[:x]
        v = version[x + 1:]
    else:
        v = version

    x = tmp.rfind(':')
    if x > 0:
        # We don't want to crash tools if someone
        # somehow got a weird nvr
        # if e != '':
        #    raise ValueError
        e = tmp[:x]
        n = tmp[x + 1:]
    else:
        n = tmp

    return n, v, r, e, a


# Preserve old function name
def split_filename(nvr):
    return splitFilename(nvr)


def dlrn_label_compare(left, right):
    warnings.warn("dlrn_label_compare() is deprecated; use label_compare().",
                  DeprecationWarning)
    if type(left) is not tuple or type(right) is not tuple:
        raise ValueError('dlrn_label_compare requires two tuples')

    l_is_dlrn = False
    r_is_dlrn = False
    l_is_rc = False
    r_is_rc = False
    l_is_ga = False
    r_is_ga = False

    lx = left[0]
    lv = left[1]
    lr = left[2]

    dlrn_regex = r'^[012](\.[0-9]{1,2})?\.[0-9]{14}\.[0-9a-f]{7}\.'

    if re.match(dlrn_regex, lr):
        l_is_dlrn = True
    elif re.match(r'^0\.[0-9]{1,2}\.0rc[123](\.|$)', lr):
        l_is_rc = True
    elif re.match(r'^0\.[0-9]{1,2}(\.|$)', lr):
        l_is_ga = True

    rx = right[0]
    rv = right[1]
    rr = right[2]

    if re.match(dlrn_regex, rr):
        r_is_dlrn = True
    elif re.match(r'^0\.[0-9]{1,2}\.0rc[123](\.|$)', rr):
        r_is_rc = True
    elif re.match(r'^0\.[0-9]{1,2}(\.|$)', rr):
        r_is_ga = True

    if r_is_dlrn == l_is_dlrn:
        return (rpmLabelCompare(left, right), False)

    altered = False
    if l_is_dlrn:
        if r_is_rc:
            # print 'fudging DLRN release for right to 1 (RC build)'
            rr = '1'
            altered = True
        if r_is_ga:
            # print 'fudging DLRN release for left to 0 (GA build)'
            lr = '0'
            altered = True
        if rv[:-1] == lv[:-1] and \
           ord(lv[-1:]) - ord(rv[-1:]) == 1:
            # print 'fudging DLRN build version (type 1l)'
            return (rpmLabelCompare((lx, rv, lr), (rx, rv, rr)), True)
        x = lv.rfind('.')
        if lv[:x] == rv:
            # 6.2-2.el6ost > 6.2.1-0.20171313131313
            # print 'fudging DLRN build version (type 2l.b)'
            return (rpmLabelCompare((lx, lv, lr), (rx, lv, rr)), True)
        if lv[:x] == rv[:x]:
            # 6.0.9-1.el7ost > 6.0.10-0.201713131313131
            lm = int(lv[x + 1:])
            rm = int(rv[x + 1:])
            if (lm - rm) == 1:
                # print 'fudging DLRN build version (type 2l)'
                return (rpmLabelCompare((lx, lv, lr), (rx, lv, rr)), True)

    if r_is_dlrn:
        if l_is_rc:
            # print 'fudging DLRN release for left to 1 (RC build)'
            lr = '1'
            altered = True
        if l_is_ga:
            # print 'fudging DLRN release for right to 0 (GA build)'
            rr = '0'
            altered = True
        if rv[:-1] == lv[:-1] and \
           ord(rv[-1:]) - ord(lv[-1:]) == 1:
            # print 'fudging DLRN build version (type 1r)'
            return (rpmLabelCompare((lx, lv, lr), (rx, lv, rr)), True)
        x = rv.rfind('.')
        if rv[:x] == lv:
            # 6.2.1-0.20171313131313 < 6.2-2.el7ost
            # print 'fudging DLRN build version (type 2r.b)'
            return (rpmLabelCompare((lx, lv, lr), (rx, lv, rr)), True)
        if lv[:x] == rv[:x]:
            # 6.0.10-0.201713131313131 < 6.0.9-1.el7ost
            lm = int(lv[x + 1:])
            rm = int(rv[x + 1:])
            if (rm - lm) == 1:
                # print 'fudging DLRN build version (type 2r)'
                return (rpmLabelCompare((lx, lv, lr), (rx, lv, rr)), True)

    # Version are equal, but maybe we altered release
    return (rpmLabelCompare((lx, lv, lr), (rx, rv, rr)), altered)


def cpaas_label_compare(left, right):
    if type(left) is not tuple or type(right) is not tuple:
        raise ValueError('cpaas_label_compare requires two tuples')
    if len(left) != 3 or len(right) != 3:
        raise ValueError('cpaas_label_compare requires two tuples of length 3')

    rx = re.compile(r'\.[0-9]+$')
    l_m = rx.search(left[2])
    r_m = rx.search(right[2])

    if l_m:
        l_r = left[2][0:l_m.span()[0]]
    else:
        l_r = left[2]

    if r_m:
        r_r = right[2][0:r_m.span()[0]]
    else:
        r_r = right[2]

    return rpmLabelCompare((left[0], left[1], l_r), (right[0], right[1], r_r))


def label_compare(left, right):
    return rpmLabelCompare(left, right)


# Preserve old method name
def labelCompare(left, right):
    return label_compare(left, right)


def drop_epoch(nevra):
    (n, v, r, e, a) = splitFilename(nevra)
    nvra = '-'.join([x for x in [n, v, r] if x != ''])
    if a:
        nvra = f'{nvra}.{a}'
    return nvra


def componentize(builds):
    if builds is None:
        return None
    if type(builds) is str:
        _builds = [builds]
    elif type(builds) is list:
        _builds = builds
    else:
        raise TypeError(
            'Builds is not str or list (' + str(type(builds)) + ')')

    ret = []
    for b in _builds:
        (n, v, r, e, a) = splitFilename(b)
        ret.append(n)

    if type(builds) is str:
        return ret[0]
    return ret



import pytest

from toolchest.hash import generate_hash


def test_good_list():
    my_list = ['openstack-foo-13.0-1', 'openstack-foo-13.0-2']
    result = generate_hash(my_list)
    expected = 'e07350662d96f'
    assert expected in result


def test_different_order_lists_return_same_hash():
    my_list = ['openstack-foo-13.0-1', 'openstack-foo-13.0-2']
    my_list2 = ['openstack-foo-13.0-2', 'openstack-foo-13.0-1']
    result1 = generate_hash(my_list)
    result2 = generate_hash(my_list2)
    assert result1 == result2


def test_different_lists_return_different_hashes():
    my_list = ['openstack-foo-13.0-1', 'openstack-foo-13.0-2']
    my_list2 = ['foo-13.0-1', 'foo-13.0-2']
    result1 = generate_hash(my_list)
    result2 = generate_hash(my_list2)
    assert result1 != result2


def test_throws_value_error_on_bad_input():
    with pytest.raises(ValueError, match='arg is not a string'):
        generate_hash(5)
    with pytest.raises(ValueError, match='arg is not a string'):
        generate_hash(None)
    with pytest.raises(ValueError, match='5 is not a string'):
        generate_hash([5])
